@@ -468,9 +468,6 @@ class StandaloneHTMLBuilder(Builder):
         else:
             self.last_updated = None
 
-        logo = path.basename(self.config.html_logo) if self.config.html_logo else ''
-        favicon = path.basename(self.config.html_favicon) if self.config.html_favicon else ''
-
         self.relations = self.env.collect_relations()
 
         rellinks: List[Tuple[str, str, str, str]] = []
@@ -513,8 +510,8 @@ class StandaloneHTMLBuilder(Builder):
             'rellinks': rellinks,
             'builder': self.name,
             'parents': [],
-            'logo': logo,
-            'favicon': favicon,
+            'logo': self.config.html_logo or '',
+            'favicon': self.config.html_favicon or '',
             'html5_doctype': html5_ready and not self.config.html4_writer,
         }
         if self.theme:
@@ -1000,16 +997,6 @@ class StandaloneHTMLBuilder(Builder):
             return uri
         ctx['pathto'] = pathto
 
-        def css_tag(css: Stylesheet) -> str:
-            attrs = []
-            for key in sorted(css.attributes):
-                value = css.attributes[key]
-                if value is not None:
-                    attrs.append('%s="%s"' % (key, html.escape(value, True)))
-            attrs.append('href="%s"' % pathto(css.filename, resource=True))
-            return '<link %s />' % ' '.join(attrs)
-        ctx['css_tag'] = css_tag
-
         def hasdoc(name: str) -> bool:
             if name in self.env.all_docs:
                 return True
@@ -1140,6 +1127,26 @@ def convert_html_js_files(app: Sphinx, config: Config) -> None:
     config.html_js_files = html_js_files  # type: ignore
 
 
+def setup_css_tag_helper(app: Sphinx, pagename: str, templatename: str,
+                         context: Dict, doctree: Node) -> None:
+    """Set up css_tag() template helper.
+
+    .. note:: This set up function is added to keep compatibility with webhelper.
+    """
+    pathto = context.get('pathto')
+
+    def css_tag(css: Stylesheet) -> str:
+        attrs = []
+        for key in sorted(css.attributes):
+            value = css.attributes[key]
+            if value is not None:
+                attrs.append('%s="%s"' % (key, html.escape(value, True)))
+        attrs.append('href="%s"' % pathto(css.filename, resource=True))
+        return '<link %s />' % ' '.join(attrs)
+
+    context['css_tag'] = css_tag
+
+
 def setup_js_tag_helper(app: Sphinx, pagename: str, templatename: str,
                         context: Dict, doctree: Node) -> None:
     """Set up js_tag() template helper.
@@ -1347,6 +1354,7 @@ def setup(app: Sphinx) -> Dict[str, Any]:
     app.connect('config-inited', validate_html_logo, priority=800)
     app.connect('config-inited', validate_html_favicon, priority=800)
     app.connect('builder-inited', validate_math_renderer)
+    app.connect('html-page-context', setup_css_tag_helper)
     app.connect('html-page-context', setup_js_tag_helper)
     app.connect('html-page-context', setup_resource_paths)
 
