@@ -4,12 +4,11 @@
 
     Toctree collector for sphinx.environment.
 
-    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-from typing import Any, Dict, List, Set, Tuple, TypeVar
-from typing import cast
+from typing import Any, Dict, List, Set, Tuple, Type, TypeVar, cast
 
 from docutils import nodes
 from docutils.nodes import Element, Node
@@ -21,12 +20,7 @@ from sphinx.environment.adapters.toctree import TocTree
 from sphinx.environment.collectors import EnvironmentCollector
 from sphinx.locale import __
 from sphinx.transforms import SphinxContentsFilter
-from sphinx.util import url_re, logging
-
-if False:
-    # For type annotation
-    from typing import Type  # for python3.5.1
-
+from sphinx.util import logging, url_re
 
 N = TypeVar('N')
 
@@ -68,9 +62,9 @@ class TocTreeCollector(EnvironmentCollector):
         docname = app.env.docname
         numentries = [0]  # nonlocal again...
 
-        def traverse_in_section(node: Element, cls: "Type[N]") -> List[N]:
+        def traverse_in_section(node: Element, cls: Type[N]) -> List[N]:
             """Like traverse(), but stay within the same section."""
-            result = []  # type: List[N]
+            result: List[N] = []
             if isinstance(node, cls):
                 result.append(node)
             for child in node.children:
@@ -81,7 +75,7 @@ class TocTreeCollector(EnvironmentCollector):
             return result
 
         def build_toc(node: Element, depth: int = 1) -> nodes.bullet_list:
-            entries = []  # type: List[Element]
+            entries: List[Element] = []
             for sectionnode in node:
                 # find all toctree nodes in this section and add them
                 # to the toc (just copying the toctree node which is then
@@ -106,7 +100,7 @@ class TocTreeCollector(EnvironmentCollector):
                         '', '', internal=True, refuri=docname,
                         anchorname=anchorname, *nodetext)
                     para = addnodes.compact_paragraph('', '', reference)
-                    item = nodes.list_item('', para)  # type: Element
+                    item: Element = nodes.list_item('', para)
                     sub_item = build_toc(sectionnode, depth + 1)
                     if sub_item:
                         item += sub_item
@@ -142,7 +136,7 @@ class TocTreeCollector(EnvironmentCollector):
         # a list of all docnames whose section numbers changed
         rewrite_needed = []
 
-        assigned = set()  # type: Set[str]
+        assigned: Set[str] = set()
         old_secnumbers = env.toc_secnumbers
         env.toc_secnumbers = {}
 
@@ -192,7 +186,7 @@ class TocTreeCollector(EnvironmentCollector):
                                       '(nested numbered toctree?)'), ref,
                                    location=toctreenode, type='toc', subtype='secnum')
                 elif ref in env.tocs:
-                    secnums = {}  # type: Dict[str, Tuple[int, ...]]
+                    secnums: Dict[str, Tuple[int, ...]] = {}
                     env.toc_secnumbers[ref] = secnums
                     assigned.add(ref)
                     _walk_toc(env.tocs[ref], secnums, depth, env.titles.get(ref))
@@ -216,14 +210,18 @@ class TocTreeCollector(EnvironmentCollector):
 
         rewrite_needed = []
 
-        assigned = set()  # type: Set[str]
+        assigned: Set[str] = set()
         old_fignumbers = env.toc_fignumbers
         env.toc_fignumbers = {}
-        fignum_counter = {}  # type: Dict[str, Dict[Tuple[int, ...], int]]
+        fignum_counter: Dict[str, Dict[Tuple[int, ...], int]] = {}
 
         def get_figtype(node: Node) -> str:
             for domain in env.domains.values():
                 figtype = domain.get_enumerable_node_type(node)
+                if domain.name == 'std' and not domain.get_numfig_title(node):  # type: ignore
+                    # Skip if uncaptioned node
+                    continue
+
                 if figtype:
                     return figtype
 
@@ -283,7 +281,7 @@ class TocTreeCollector(EnvironmentCollector):
                 _walk_doctree(docname, doctree, secnum)
 
         if env.config.numfig:
-            _walk_doc(env.config.master_doc, tuple())
+            _walk_doc(env.config.root_doc, tuple())
             for docname, fignums in env.toc_fignumbers.items():
                 if fignums != old_fignumbers.get(docname):
                     rewrite_needed.append(docname)
