@@ -64,7 +64,7 @@ RoleFunction = Callable[[str, str, str, int, Inliner, Dict[str, Any], List[str]]
                         Tuple[List[nodes.Node], List[nodes.system_message]]]
 
 # A option spec for directive
-OptionSpec = Dict[str, Callable[[Optional[str]], Any]]
+OptionSpec = Dict[str, Callable[[str], Any]]
 
 # title getter functions for enumerable nodes (see sphinx.domains.std)
 TitleGetter = Callable[[nodes.Node], str]
@@ -105,27 +105,30 @@ def restify(cls: Optional[Type]) -> str:
     """Convert python class to a reST reference."""
     from sphinx.util import inspect  # lazy loading
 
-    if cls is None or cls is NoneType:
-        return ':obj:`None`'
-    elif cls is Ellipsis:
-        return '...'
-    elif cls in INVALID_BUILTIN_CLASSES:
-        return ':class:`%s`' % INVALID_BUILTIN_CLASSES[cls]
-    elif inspect.isNewType(cls):
-        return ':class:`%s`' % cls.__name__
-    elif types_Union and isinstance(cls, types_Union):
-        if len(cls.__args__) > 1 and None in cls.__args__:
-            args = ' | '.join(restify(a) for a in cls.__args__ if a)
-            return 'Optional[%s]' % args
+    try:
+        if cls is None or cls is NoneType:
+            return ':obj:`None`'
+        elif cls is Ellipsis:
+            return '...'
+        elif cls in INVALID_BUILTIN_CLASSES:
+            return ':class:`%s`' % INVALID_BUILTIN_CLASSES[cls]
+        elif inspect.isNewType(cls):
+            return ':class:`%s`' % cls.__name__
+        elif types_Union and isinstance(cls, types_Union):
+            if len(cls.__args__) > 1 and None in cls.__args__:
+                args = ' | '.join(restify(a) for a in cls.__args__ if a)
+                return 'Optional[%s]' % args
+            else:
+                return ' | '.join(restify(a) for a in cls.__args__)
+        elif cls.__module__ in ('__builtin__', 'builtins'):
+            return ':class:`%s`' % cls.__name__
         else:
-            return ' | '.join(restify(a) for a in cls.__args__)
-    elif cls.__module__ in ('__builtin__', 'builtins'):
-        return ':class:`%s`' % cls.__name__
-    else:
-        if sys.version_info >= (3, 7):  # py37+
-            return _restify_py37(cls)
-        else:
-            return _restify_py36(cls)
+            if sys.version_info >= (3, 7):  # py37+
+                return _restify_py37(cls)
+            else:
+                return _restify_py36(cls)
+    except (AttributeError, TypeError):
+        return repr(cls)
 
 
 def _restify_py37(cls: Optional[Type]) -> str:
@@ -153,6 +156,7 @@ def _restify_py37(cls: Optional[Type]) -> str:
         else:
             text = restify(cls.__origin__)
 
+        origin = getattr(cls, '__origin__', None)
         if not hasattr(cls, '__args__'):
             pass
         elif all(is_system_TypeVar(a) for a in cls.__args__):
@@ -161,6 +165,8 @@ def _restify_py37(cls: Optional[Type]) -> str:
         elif cls.__module__ == 'typing' and cls._name == 'Callable':
             args = ', '.join(restify(a) for a in cls.__args__[:-1])
             text += r"\ [[%s], %s]" % (args, restify(cls.__args__[-1]))
+        elif cls.__module__ == 'typing' and getattr(origin, '_name', None) == 'Literal':
+            text += r"\ [%s]" % ', '.join(repr(a) for a in cls.__args__)
         elif cls.__args__:
             text += r"\ [%s]" % ", ".join(restify(a) for a in cls.__args__)
 
@@ -294,7 +300,7 @@ def stringify(annotation: Any) -> str:
         else:
             return '.'.join([annotation.__module__, annotation.__name__])
     elif inspect.isNewType(annotation):
-        # Could not get the module where it defiend
+        # Could not get the module where it defined
         return annotation.__name__
     elif not annotation:
         return repr(annotation)
@@ -362,6 +368,9 @@ def _stringify_py37(annotation: Any) -> str:
             args = ', '.join(stringify(a) for a in annotation.__args__[:-1])
             returns = stringify(annotation.__args__[-1])
             return '%s[[%s], %s]' % (qualname, args, returns)
+        elif qualname == 'Literal':
+            args = ', '.join(repr(a) for a in annotation.__args__)
+            return '%s[%s]' % (qualname, args)
         elif str(annotation).startswith('typing.Annotated'):  # for py39+
             return stringify(annotation.__args__[0])
         elif all(is_system_TypeVar(a) for a in annotation.__args__):
