@@ -8,7 +8,6 @@
     :license: BSD, see LICENSE for details.
 """
 
-import warnings
 from typing import Any, Dict, Iterable, cast
 
 from docutils import nodes
@@ -18,7 +17,6 @@ from docutils.writers.manpage import Writer
 
 from sphinx import addnodes
 from sphinx.builders import Builder
-from sphinx.deprecation import RemovedInSphinx40Warning
 from sphinx.locale import _, admonitionlabels
 from sphinx.util import logging
 from sphinx.util.docutils import SphinxTranslator
@@ -61,13 +59,16 @@ class NestedInlineTransform:
         for node in self.document.traverse(matcher):  # type: TextElement
             if any(matcher(subnode) for subnode in node):
                 pos = node.parent.index(node)
-                for subnode in reversed(node[1:]):
+                for subnode in reversed(list(node)):
                     node.remove(subnode)
                     if matcher(subnode):
                         node.parent.insert(pos + 1, subnode)
                     else:
                         newnode = node.__class__('', '', subnode, **node.attributes)
                         node.parent.insert(pos + 1, newnode)
+                # move node if all children became siblings of the node
+                if not len(node):
+                    node.parent.remove(node)
 
 
 class ManualPageTranslator(SphinxTranslator, BaseTranslator):
@@ -75,16 +76,9 @@ class ManualPageTranslator(SphinxTranslator, BaseTranslator):
     Custom translator.
     """
 
-    _docinfo = {}  # type: Dict[str, Any]
+    _docinfo: Dict[str, Any] = {}
 
-    def __init__(self, *args: Any) -> None:
-        if isinstance(args[0], nodes.document) and isinstance(args[1], Builder):
-            document, builder = args
-        else:
-            warnings.warn('The order of arguments for ManualPageTranslator has been changed. '
-                          'Please give "document" as 1st and "builder" as 2nd.',
-                          RemovedInSphinx40Warning, stacklevel=2)
-            builder, document = args
+    def __init__(self, document: nodes.document, builder: Builder) -> None:
         super().__init__(document, builder)
 
         self.in_productionlist = 0
@@ -129,6 +123,13 @@ class ManualPageTranslator(SphinxTranslator, BaseTranslator):
     def depart_start_of_file(self, node: Element) -> None:
         pass
 
+    #############################################################
+    # Domain-specific object descriptions
+    #############################################################
+
+    # Top-level nodes for descriptions
+    ##################################
+
     def visit_desc(self, node: Element) -> None:
         self.visit_definition_list(node)
 
@@ -148,6 +149,27 @@ class ManualPageTranslator(SphinxTranslator, BaseTranslator):
     def depart_desc_signature_line(self, node: Element) -> None:
         self.body.append(' ')
 
+    def visit_desc_content(self, node: Element) -> None:
+        self.visit_definition(node)
+
+    def depart_desc_content(self, node: Element) -> None:
+        self.depart_definition(node)
+
+    def visit_desc_inline(self, node: Element) -> None:
+        pass
+
+    def depart_desc_inline(self, node: Element) -> None:
+        pass
+
+    # Nodes for high-level structure in signatures
+    ##############################################
+
+    def visit_desc_name(self, node: Element) -> None:
+        pass
+
+    def depart_desc_name(self, node: Element) -> None:
+        pass
+
     def visit_desc_addname(self, node: Element) -> None:
         pass
 
@@ -166,12 +188,6 @@ class ManualPageTranslator(SphinxTranslator, BaseTranslator):
     def depart_desc_returns(self, node: Element) -> None:
         pass
 
-    def visit_desc_name(self, node: Element) -> None:
-        pass
-
-    def depart_desc_name(self, node: Element) -> None:
-        pass
-
     def visit_desc_parameterlist(self, node: Element) -> None:
         self.body.append('(')
         self.first_param = 1
@@ -200,11 +216,7 @@ class ManualPageTranslator(SphinxTranslator, BaseTranslator):
     def depart_desc_annotation(self, node: Element) -> None:
         pass
 
-    def visit_desc_content(self, node: Element) -> None:
-        self.visit_definition(node)
-
-    def depart_desc_content(self, node: Element) -> None:
-        self.depart_definition(node)
+    ##############################################
 
     def visit_versionmodified(self, node: Element) -> None:
         self.visit_paragraph(node)
